/*pimp.c -- main source for popmail retrieval client         
 * $Id: pimp.c,v 1.1 1996/04/08 15:49:50 gh Exp $
 */

/*
    Copyright (C) 1996 Robert Mark Waugh (rmw@io.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <pwd.h>

#include <time.h>

#include "config.h"
#include "pimp.h"

time_t 
elapsed ()
{
  return time (NULL) - starttime;
}

int 
main (int argc, char **argv)
{
  struct userlist *root;
  starttime = time (NULL);
  if (argc == 1)		/* if argc is 1, open std config file */
    {
      if (read_config (config, rtime, 0) == -1)
	{
	  fprintf (stderr, "%s", BANNER);
	  fprintf (stderr, "usage: %s <-f file|hostname localname remotename password [port]>\n", argv[0]);
	  fprintf (stderr, "could not open %s\n", config);
	  exit (EXIT_FAILURE);
	}
      else
	exit (0);
    }
  if (argc == 3)		/* if not, look for a command line defined config */
    {
      if (!(strcmp (argv[1], "-f")))
	{
	  if (read_config (argv[2], rtime, 1) == -1)
	    {
	      fprintf (stderr, "%s", BANNER);
	      fprintf (stderr, "usage: %s <-f file|hostname localname remotename password [port]>\n", argv[0]);
	      fprintf (stderr, "could not open %s\n", argv[2]);
	      exit (EXIT_FAILURE);
	    }
	  else
	    exit (0);
	}
      else
	/* got left off a arg or two */
	{
	  fprintf (stderr, "%s", BANNER);
	  fprintf (stderr, "usage: %s <-f file|hostname localname remotename password [port]>\n", argv[0]);
	  exit (EXIT_FAILURE);
	}
    }
  if (argc < 5 || argc > 6)	/* too many or too little */
    {
      fprintf (stderr, "%s", BANNER);
      fprintf (stderr, "usage: %s <-f file|hostname localname remotename password [port]>\n", argv[0]);
      exit (EXIT_FAILURE);
    }
  else
    /* right number, we can only hope they got the sequence right */
    {
      root = (struct userlist *) malloc (sizeof (struct userlist));
      root->server = strdup (argv[1]);	/* dupicate argv's. make struct to hold these */
      root->localname = strdup (argv[2]);
      root->remotename = strdup (argv[3]);
      root->password = strdup (argv[4]);
      root->next = NULL;
      if (argc == 6)		/* if the port is defined, try to get it */
	{
	  if ((atoi (argv[5])) == (int) NULL)	/* port position is not a number */
	    {
	      fprintf (stderr, "%s", BANNER);
	      fprintf (stderr, "usage: %s <-f file|hostname localname remotename password [port]>\n", argv[0]);
	      fprintf (stderr, "port %s -- not a valid port number\n", argv[5]);
	      exit (EXIT_FAILURE);
	    }
	  root->port = atoi (argv[5]);
	}
      else
	root->port = POP3_PORT;
    }
  printf ("%s\n", BANNER);	/* ready to start grab */

#ifdef DEFINE
  fprintf (stderr, "Command line args used\n");
#endif
  pop3 (root, 1, rtime);	/* start main grab */
  printf ("\nTime elapsed: %i\n", (int) elapsed ());
  return 1;
}
