/* The only possible thing that you may want to modify is the max_line_len
 * which you may want to adjust depending on your performance.
 * Unless you're really really worried about how much memory it's using
 * 20k is probably the best possible figure.  Most of the time you'll
 * not use all that space, so you will waste a few kb of memory, but,
 * you will find a signficant performance gain.  If you don't have
 * enough memory to handle a single 20k buffer, then you probably
 * don't want to use this program.
 */
/*
    Copyright (C) 1996 Robert Mark Waugh (rmw@io.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define VERSION		"1.6"
#define BANNER		"pimp " VERSION " - GPL 1996 Robert Mark Waugh (rmw@io.com)\n"

#define OK			"+OK"
#define POP_OK			"+OK"
#define MAX_LINE_LEN		20480
#define MAX_HOSTNAME_LEN	256
#define MAX_PORT_LEN		5
#define POP3_PORT		110
#define MAX_USERNAME_LEN	65

#ifndef INADDR_NONE
#ifdef INADDR_BROADCAST
#define INADDR_NONE INADDR_BROADCAST
#else
#define INADDR_NONE -1
#endif
#endif

struct maillist
  {
    int number;
    unsigned long mail_length;
    struct maillist *next;
  };

struct userlist
  {
    char *server, *localname, *remotename, *password;
    int port;
    struct userlist *next;
  };

/* misc debugging globals */

/* Function prototypes */

time_t elapsed ();
int start_connection (const char *host, unsigned short int port);
int probesocket (int sock, int seconds, int useconds);
char *get_data (int sock, int probesec, int probeusec);
void send_data (int sock, const char *buffer);
int check_reply (const char *buffer);
int UserAuth (int sock, char *rname, char *passwd);
struct maillist *build_list (int sock);
void close_connection (int sock);
int pop3 (struct userlist *userroot, int number, char *rtime);
int read_config (char *config, char *rtime, int specified);
void retrieve_message (int sock, unsigned long length, FILE * spoolfd, char *rtime);
char *format_mail (char *mailbody, char *rtime);

/* End of prototypes */
