/* pimprc.c -- reads and parses resource files, handles security etc
 * $Id: pimprc.c,v 1.1 1996/04/08 15:49:50 gh Exp $
 */ 

/*
    Copyright (C) 1996 Robert Mark Waugh (rmw@io.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stddef.h>
#include <sys/types.h>

#include "pimp.h"

extern char *config;
extern int pop3_port;
extern int mail_grp;
extern char *spool;
extern char *rtime;

/* read_config()
 * uses: char *config, char *rtime, int specified
 * returns: int
 * notes: can exit, will modify later
 * this function needs to be broken down a bit
 * reads conf file if exists, returns -1 if not.
 * invokes pop3() internally.  May want to avoid this.
 * Adds another function layer that never is returned from.
 *
 */
int 
read_config (char *config, char *rtime, int specified)
{
  char line[MAX_LINE_LEN];
  struct userlist *root, *current;
  int number = 0, check = 0;
  char *path;
  FILE *cfg;
  char *buffer, *portbuffer;
  struct stat statbuf;

/* this checks for conf file security.  This is a messy hack.  I wasn't
 * feeling good when I wrote this, but I figured it needed to be done.
 * Now, I'm going to go pee before I splode.
 */
/* here's how it works:
 * If there wasn't a conf file specified, look for one in the users
 * home area.  If the HOME env is not set, it has to be the system.
 * Otherwise, use the HOME one.
 * If, however, the user who is invoking pimp is the owner of the
 * system file, the system file _must_ be used.
 * This can be avoided by specifying the file on the command line.
 */
  if (!specified)
    {
      if (getenv ("HOME") == NULL)
	path = config;
      else
	{
	  path = (char *) alloca (strlen (getenv ("HOME")) * strlen ("/.poprc") + 1);
	  strcpy (path, getenv ("HOME"));
	  strcat (path, "/.poprc");
	}
      if (lstat (config, &statbuf) < 0)
	;
      else
	{
	  if (statbuf.st_uid == getuid ())
	    {
	      path = config;
	    }
	}

    }
  else
    path = config;
  if (lstat (path, &statbuf) < 0)
    {
      perror (NULL);
      exit (EXIT_FAILURE);
    }
  if ((statbuf.st_mode & S_IFLNK) == S_IFLNK)
    {
      fprintf (stderr, "%s: symbolic link.\n", path);
      exit (EXIT_FAILURE);
    }
  if (statbuf.st_mode & ~(S_IFREG | S_IREAD | S_IWRITE))
    {
      fprintf (stderr, "%s: file permissions too great.\n", path);
      exit (EXIT_FAILURE);
    }
  if (statbuf.st_uid != getuid ())
    {
      fprintf (stderr, "%s: not owner\n", path);
      exit (EXIT_FAILURE);
    }
  cfg = fopen (path, "r");
  if (cfg == NULL)
    {
      return -1;
    }
  root = (struct userlist *) malloc (sizeof (struct userlist));
  current = root;
  while (fgets (line, MAX_LINE_LEN - 1, cfg) != NULL)
    {
      check = 0;
      if (number)
	{
	  current->next = (struct userlist *) malloc (sizeof (struct userlist));
	  current = current->next;
	}
      buffer = (char *) strdup (line);
      if ((current->server = strdup (strtok (buffer, " "))) == NULL)
	{
	  fprintf (stderr, "%s: error reading hostname\n", path);
	  exit (EXIT_FAILURE);
	}
      else if ((current->server[0] == '#'))
	{
	  free (current->server);
	  continue;
	}
      else
	check++;
      if ((current->localname = strdup (strtok (NULL, " "))) == NULL)
	{
	  fprintf (stderr, "%s: error reading localname\n", path);
	  exit (EXIT_FAILURE);
	}
      else
	check++;
      if ((current->remotename = strdup (strtok (NULL, " "))) == NULL)
	{
	  fprintf (stderr, "%s: error reading remotename\n", path);
	  exit (EXIT_FAILURE);
	}
      else
	check++;
      if ((current->password = strdup (strtok (NULL, " "))) == NULL)
	{
	  fprintf (stderr, "%s: error reading password\n", path);
	  exit (EXIT_FAILURE);
	}
      else
	check++;
      if ((portbuffer = strtok (NULL, " ")) != NULL)
	if ((current->port = atoi (portbuffer)) == (int) NULL)
	  {
	    fprintf (stderr, "%s: port is not number\n", path);
	    exit (EXIT_FAILURE);
	  }
	else
	  current->port = POP3_PORT;
      free (buffer);
      if (check != 4)
	{
	  fprintf (stderr, "Entry does not contain enough information.\n");
	  exit (EXIT_FAILURE);
	}
      current->next = NULL;
      number++;
    }
  fclose (cfg);
  if (number != 0)
    pop3 (root, number, rtime);
  return 0;
}
