/* pop3.c -- main protocol driver                          
 * $Id: pop3.c,v 1.1 1996/04/08 15:49:50 gh Exp $
 */
/*

    Copyright (C) 1996 Robert Mark Waugh (rmw@io.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <pwd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "pimp.h"

extern char *config;
extern int pop3_port;
extern char *spool;
extern int mail_grp;
extern char *rtime;
const char ok[3] = "+OK";	/*
				   Good reply 
				 */
struct in_addr saddr;		/*
				   network stuff 
				 */

/*
   used to start our connection... returns fd 
 */
int
start_connection (const char *host, unsigned short int port)
{
  int sock;
  unsigned long inaddr;
  struct sockaddr_in ad;
  struct hostent *hp;

  memset (&ad, 0, sizeof (ad));
  ad.sin_family = AF_INET;

  inaddr = inet_addr (host);
  hp = gethostbyname (host);
  if (hp == NULL)
    return -1;
  if (inaddr != INADDR_NONE)
    memcpy (&ad.sin_addr, &inaddr, sizeof (inaddr));
  else
    {
      memcpy (&ad.sin_addr, hp->h_addr, hp->h_length);
    }
  ad.sin_port = htons (port);

  sock = socket (AF_INET, SOCK_STREAM, 0);
  if (sock < 0)
    return sock;
  if (connect (sock, (struct sockaddr *) &ad, sizeof (ad)) < 0)
    return -1;
  printf ("Connected to %s.\n", hp->h_name);
  return sock;

}

/*
   This is an interesting function.
   * This function uses select to probe a socket to see if it's got data
   * pending in the buffer if it's a blocking socket.  If you read the
   * select() man page, it'll become less obvious why I found this function
   * so interesting.  If you don't see why I find this function interesting,
   * go away.
 */
int
probesocket (int sock, int seconds, int useconds)
{

  fd_set read_fd_stat;
  struct timeval tv;
  int status = 0;

#ifdef DEBUG
  fprintf (stderr, "Setting up to probe socket:%i\n", elapsed ());
#endif
  FD_ZERO (&read_fd_stat);
  FD_SET (sock, &read_fd_stat);

  tv.tv_sec = seconds;
  tv.tv_usec = useconds;

  status = select ((sock + 1), &read_fd_stat, NULL, NULL, &tv);
#ifdef DEBUG
  fprintf (stderr, "Select finished:%i\n", elapsed ());
#endif
  return status;
}

/*
   * This is, IMHO, another interesting function.
   * Ideally, this function reads all the pending data in a socket.
   * It should read until there is no more to read and then it'll wait
   * for the passed time limit.  If there is no more it'll return
   * a pointer to all the read data from the socket, unparsed and
   * contiguous.
 */
char *
get_data (int sock, int probesec, int probeusec)
{
  int readlen = 0, length = 0, tenthseconds = 0;
  char *data, *readbuffer;

#ifdef DEBUG
  fprintf (stderr, "Beginning get data:%i\n", elapsed ());
#endif
  data = (char *) malloc (1);
  readbuffer = (char *) malloc (MAX_LINE_LEN);
  readbuffer[0] = data[0] = '\0';
  do
    {
      tenthseconds = 0;
      while (tenthseconds != 900)
	{
	  if (probesocket (sock, 0, 100000) > 0)
	    break;
	  tenthseconds++;
	}
      if (tenthseconds == 900)
	{
	  fprintf (stderr, "130 seconds for a read is too long.\n");
	  exit (EXIT_FAILURE);
	}
/****************************************
  	readbuffer = (char *) realloc(readbuffer, (MAX_LINE_LEN+strlen(readbuffer)+1));
*****************************************/
      readlen = read (sock, readbuffer, MAX_LINE_LEN - 1);
      if (readlen < 0)
	{
	  perror ("socket: reading data");
	  exit (EXIT_FAILURE);
	}
      readbuffer[readlen] = '\0';
      data = (char *) realloc (data, length + readlen + 1);
      strcpy (data + length, readbuffer);
      length = readlen + length;
    }
  while (!(data[length - 3] == '.' && data[length - 2] == '\r'));
#ifdef DEBUG
  fprintf (stderr, "Done with get: %i:\n", elapsed ());
  fprintf (stderr, "Returning string:\n%s", data);
#endif
  free (readbuffer);
  return data;
}

/*
   This function sends all the data in a buffer to the socket
   * there is nothing interesting about this function.
 */
void
send_data (int sock, const char *buffer)
{
  int err;

#ifdef DEBUG
  fprintf (stderr, "Sent:%i octets:elapsed:\n", (strlen (buffer)), elapsed ());
#endif
  err = write (sock, buffer, (strlen (buffer)));
  if (err == EOF)
    {
      perror ("socket: writing data");
      exit (EXIT_FAILURE);
    }
#ifdef DEBUG
  fprintf (stderr, "Sent data: %i:%i\n%s", err, elapsed (), buffer);
#endif
}

/*
   this function just checks the server's reply for an error.
   * if there is one, pull out the error and show it to whoever
   * cares.  if there isn't and it's _not_ ok, this is probably
   * a non-standard POP3 server.
 */
int
check_reply (const char *buffer)
{
  char *reply_error, *reply_buffer;

/*
   * for some reason some pop servers do not satisfy standard string
   * searching routines.  This does not make sense.
   * So, I check each individual fucking character to see if I got an
   * +OK. This should be resolved when I parse with a lex/yacc parser.
 */
  if (buffer[0] != '+' && buffer[1] != 'O' && buffer[2] != 'K')
    {
      reply_buffer = (char *) strdup (buffer);
      if ((reply_error = strtok (reply_buffer, "-ERR")) == NULL)
	fprintf (stderr, "POP server: reply not understood: %s\n", buffer);
      else
	fprintf (stderr, "POP server error: %s\n", reply_error);
      exit (EXIT_FAILURE);
    }
  return 1;
}

int
UserAuth (int sock, char *rname, char *passwd)
{
  int returnval = 1;
  char *buffer, *sendbuffer;

#ifdef DEBUG
  fprintf (stderr, "Sending USER %s:%i\n", rname, elapsed ());
#endif
  sendbuffer = (char *) malloc (strlen ("USER ") + strlen (rname) + 2);
  sprintf (sendbuffer, "USER %s\n", rname);
  send_data (sock, sendbuffer);
  free (sendbuffer);

  buffer = get_data (sock, 2, 0);
  returnval = check_reply (buffer);
  free (buffer);

#ifdef DEBUG
  fprintf (stderr, "Sending PASS %s:%i\n", passwd, elapsed ());
#endif
  sendbuffer = (char *) malloc (strlen ("PASS ") + strlen (passwd) + 2);
  sprintf (sendbuffer, "PASS %s\n", passwd);
  send_data (sock, sendbuffer);
  free (sendbuffer);

  buffer = get_data (sock, 2, 0);
  check_reply (buffer);
  free (buffer);
  printf ("Accepted username and password.\n");
  fflush (stdout);
  return returnval;
}

struct maillist *
build_list (int sock)
{
  int sequence = 0;
  char *buffer, *listbuffer;
  struct maillist *root, *current;
  send_data (sock, "LIST\n");
  buffer = get_data (sock, 3, 0);
#ifdef DEBUG
  fprintf (stderr, "LIST:\n%s\nelapsed:%i\n", buffer, elapsed ());
#endif
  listbuffer = strtok (buffer, "\n");
  root = (struct maillist *) malloc (sizeof (struct maillist));
  current = root;

  while ((listbuffer = strtok (NULL, "\n")) != NULL)
    {
      if (listbuffer[0] == '.')
	{
#ifdef DEBUG
	  fprintf (stderr, "No more mail to put in list.:%i\n", elapsed ());
#endif
	  if (!sequence)
	    return NULL;
	  break;
	}
      current->mail_length = (atoi (strrchr (listbuffer, ' ')));
      current->number = ++sequence;
#ifdef DEBUG
      fprintf (stderr, "current->mail_length = %u:%i:elapsed:\n", current->mail_length,
	       current->number, elapsed ());
#endif
      current->next = (struct maillist *) malloc (sizeof (struct maillist));
      current = current->next;
      current->mail_length = 0;
      current->number = 0;
      current->next = NULL;
    }
  free (buffer);
#ifdef DEBUG
  malloc_dump ();
#endif
  return root;
}

/*
   this one is not interesting 
 */

void
close_connection (int sock)
{
  send_data (sock, "QUIT\n");
  close (sock);
}

void
clear_maillist (struct maillist *root)
{
  struct maillist *current, *next;

  current = root;
  while (current != NULL)
    {
      next = current->next;
      free (current);
      current = next;
    }
}
void 
clear_userlist (struct userlist *root)
{
  struct userlist *current, *next;

  current = root;
  while (current != NULL)
    {
      next = current->next;
      free (current);
      current = next;
    }
}

/*
   this is the main protocol function.
   * it's currently very gross.
 */
int
pop3 (struct userlist *userroot, int number, char *rtime)
{
  extern void retrieve_message ();
  int i, total = 0, userentry;
  FILE *spoolfd;
  char *buffer, *sendbuffer;
  char spoolfile[MAX_LINE_LEN];
  int sock, fd;
  struct maillist *mailroot, *mailcurrent;
  struct userlist *usercurrent;
  time_t curtime;

  extern char *spool;
  extern int mail_grp;

  userentry = 1;
  usercurrent = userroot;
  while (userentry <= number)
    {
      sock = start_connection (usercurrent->server, usercurrent->port);
      if(sock < 0)
	{
	 perror(NULL);
	 exit(EXIT_FAILURE);
	}
      /*
         connect to host and port 
       */
#ifdef DEBUG
      fprintf (stderr, "starting first get: %i\n", elapsed ());
#endif

/*
   After connection see if the server greated us. 
 */
      buffer = get_data (sock, 2, 0);
#ifdef DEBUG
      fprintf (stderr, "Greeting: %s\n", buffer);
#endif
      check_reply (buffer);
      free (buffer);

/*
   see if we're authorized to get into this mailbody 
   UserAuth always exits or returns 1, so this check is 
   currently redundant.  I want to get away from high level 
   functions handling errors. 
 */
      if (!UserAuth (sock, usercurrent->remotename, usercurrent->password))
	exit (EXIT_FAILURE);
      fflush (stdout);

/*
   yay we made it, now get a list of waiting mail. 
   number of mails waiting extracted by number of replies, 
   length of mail body used for optimal read from socket. 
 */
      mailroot = build_list (sock);

      if (mailroot == NULL)
	{
	  printf ("No mail.\n");
	  if (number > 1)
	    sleep (2);
	  userentry++;
	  usercurrent = usercurrent->next;
	  putchar ('\n');
	  close_connection (sock);
	  continue;
	}
      sprintf (spoolfile, "%s/%s", spool, usercurrent->localname);

      i = 1;
      mailcurrent = mailroot;
      sendbuffer = (char *) malloc (sizeof (char) * strlen ("RETR ") + 7);
      sendbuffer[0] = '\0';
      fd = open (spoolfile, (O_WRONLY | O_APPEND | O_CREAT), (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP));
      if (fd == -1)
	{
	  perror ("opening spool file");
	  exit (EXIT_FAILURE);
	}

      spoolfd = fdopen (fd, "a");
      curtime = time (NULL);
      rtime = asctime (localtime (&curtime));

      while (mailcurrent->number != 0)
	{

	  sprintf (sendbuffer, "RETR %i\n", mailcurrent->number);
	  send_data (sock, sendbuffer);

#ifdef DEBUG
	  fprintf (stderr, "Retrieving mail %i:%i\n", current->number, elapsed ());
#endif

	  retrieve_message (sock, mailcurrent->mail_length, spoolfd, rtime);
#ifdef DELETION
	  sendbuffer = (char *) malloc ((sizeof (char)) * (strlen ("DELE ") + 2));
	  sprintf (sendbuffer, "DELE %d\n", mailcurrent->number);
	  send_data (sock, sendbuffer);
	  buffer = get_data (sock, 3, 0);
	  check_reply (buffer);
#endif
	  mailcurrent = mailcurrent->next;
	  if (!(i % 20))
	    {
	      fflush (stdout);
	      fflush (spoolfd);
	      putchar ('#');
	    }
	  else
	    putchar ('.');
	  i++;
	}
      free (sendbuffer);
      userentry++;
      usercurrent = usercurrent->next;
      putchar ('\n');
      fclose (spoolfd);
      close (fd);
      close_connection (sock);
      fflush (stdout);
      if (number > 1)
	sleep (2);
      clear_maillist (mailroot);
      total = i + total;
    }
  clear_userlist (userroot);
  if (total > 0)
    printf ("\nRetrieved %i mails in %i seconds: %3.2f/sec\n", total, (int) elapsed (),
	    (float) total / (float) elapsed ());
  return (total);
}
